/**
 * Title:	    Graph
 * Copyright:	    Copyright (c) Jan-Pascal van Best
 * Email:		janpascal@vanbest.org
 * Author:	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.graph;


import java.util.*;
import java.lang.*;


/**
 * A simple implementation of the {@link Node} interface. This class implements
 * the {@link #isDeletable}/{@link #setDeletable} stuff. It does not
 * implement the <code>getName</code>, <code>getLabel</code>,
 * <code>getPropertyNames</code> and <code>getProperty</code> methods of
 * the <code>Node</code> interface.
 *
 * @author	    Jan-Pascal van Best
 * @version	1.0
 */

public class AbstractNode implements Node {

    /** Flag to indicate whether or not the node may be deleted. */
    private boolean deletable = true;
    protected double x = POSITION_UNKNOWN;
    protected double y = POSITION_UNKNOWN;

    public String name;
    public String label;
    public AbstractNode() {
        this.name = "";
        this.label = "";
    }

    public AbstractNode(String name) {
        this.name = name;
        this.label = "";
    }

    public AbstractNode(String name, String label) {
        this.name = name;
        this.label = label;
    }

    public boolean isDeletable() {
        return deletable;
    }

    public void setDeletable(boolean deletable) {
        this.deletable = deletable;
    }

    /**
     * Gets the name of this <code>Node</code>.
     * @return a <code>String</code> representing the name of this node
     */
    public String getName() {
        return name;
    }

    public String getLabel() {
        return label;
    }

    public List getPropertyNames() {
        return new ArrayList();
    }

    public String getProperty(String name) {
        return null;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }

    public void setX(double x) {
        this.x = x;
    }

    public void setY(double y) {
        this.y = y;
    }

    public void setPosition(double x, double y) {
        this.x = x;
        this.y = y;
    }

    public String toString() {
        return name;
    }
}

