// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.graph;


import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import java.text.*;
import java.util.*;
import nl.tudelft.tbm.graph.*;
import java.util.logging.*;


public class EdgeComponent extends JPanel {
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.graph");
	public final static int STYLE_STRAIGHT = 1;
	public final static int STYLE_CURVED = 2;

    Edge edge;
    private NodeComponent from;
    private NodeComponent to;
    private Paint paint = Color.RED;

    private GeneralPath shape;
    int arrowSize;
	int style;

    public EdgeComponent(Edge edge, NodeComponent from, NodeComponent to, int arrowSize, int style) {
        super();
        this.edge = edge;
        this.from = from;
        this.to = to;
        this.arrowSize = arrowSize;
		this.style = style;
        //	  setBackground(Color.WHITE);
        setOpaque(false);
        //	  setSize(500,500);
    }

    public void setPaint(Paint paint) {
        this.paint = paint;
    }

	public void setStyle(int style) {
		this.style = style;
		repaint();
	}

    public Paint getPaint() {
        return this.paint;
    }

    public void paintComponent(Graphics g) {
        //	  logger.info("Entering EdgeComponent.paintComponent()");
        super.paintComponent(g);
        Graphics2D g2 = (Graphics2D) g;

        double angle = Math.atan2(to.getY() - from.getY(), to.getX() - from.getX());

        double distance = Math.sqrt((from.getX() - to.getX()) * (from.getX() - to.getX()) + (from.getY() - to.getY()) * (from.getY() - to.getY()));
        Point2D.Double start = new Point2D.Double(from.getX() + from.size / 2 + from.size / 2 * Math.cos(angle), from.getY() + from.size / 2 + from.size / 2 * Math.sin(angle));
        Point2D.Double end = new Point2D.Double(to.getX() + to.size / 2 - to.size / 2 * Math.cos(angle), to.getY() + to.size / 2 - to.size / 2 * Math.sin(angle));
        double arrowAngle = Math.PI / 4 - angle;
        Point2D.Double rightArrow = new Point2D.Double(end.x - arrowSize * Math.sin(arrowAngle), end.y - arrowSize * Math.cos(arrowAngle));
        Point2D.Double leftArrow = new Point2D.Double(end.x - arrowSize * Math.cos(arrowAngle), end.y + arrowSize * Math.sin(arrowAngle));

        shape = new GeneralPath();
        shape.moveTo((float) start.x, (float) start.y); // Starting point
		switch (style) {
		case STYLE_STRAIGHT:
	        shape.lineTo((float) end.x, (float) end.y); // Draw main line
			break;
		case STYLE_CURVED:
			// Calculate midpoint for curve
			Point2D.Double curve = new Point2D.Double((end.x-start.x)/2 + start.x - 2*arrowSize*Math.sin(angle), (end.y-start.y)/2 + start.y + arrowSize*Math.cos(angle));
			shape.curveTo((float)curve.x, (float)curve.y, (float)curve.x, (float)curve.y, (float) end.x, (float) end.y);
			break;
		}
        shape.lineTo((float) leftArrow.x, (float) leftArrow.y); // Draw left side of arrow
        shape.moveTo((float) end.x, (float) end.y);
        shape.lineTo((float) rightArrow.x, (float) rightArrow.y); // Draw right side of arrow
        if (edge.getBidir()) {
            rightArrow = new Point2D.Double(start.x + arrowSize * Math.sin(arrowAngle), start.y - arrowSize * Math.cos(arrowAngle));
            leftArrow = new Point2D.Double(start.x + arrowSize * Math.cos(arrowAngle), start.y + arrowSize * Math.sin(arrowAngle));
            shape.moveTo((float) leftArrow.x, (float) leftArrow.y); 
            shape.lineTo((float) start.x, (float) start.y); // Draw left side of arrow
            shape.lineTo((float) rightArrow.x, (float) rightArrow.y); // Draw right side of arrow
        }
        g2.setPaint(paint);
        g2.draw(shape);

        //	  logger.info("x,y,x,y: "+(from.getX()+from.getWidth()/2)+","+(from.getY()+from.getHeight()/2)+","+(to.getX()+from.getWidth()/2)+","+(to.getY()+to.getHeight()/2));

        //	  g2.drawLine(from.getX()+from.getWidth()/2,from.getY()+from.getHeight()/2,to.getY()+from.getWidth()/2,to.getY()+to.getHeight()/2);
        //	  g2.fillOval(0,0,5,5);
    }

    public boolean contains(int x, int y) {
        return shape != null && shape.contains(x, y);
    }
}

