// This program is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.graph;

import java.text.*;
import java.util.*;
import java.util.logging.*;

public class EdgeCountFilter implements GraphFilter {
    public final static int DONT_FILTER = 0;
    public final static int ALLOW_EQUAL = 1;
    public final static int ALLOW_UNEQUAL = 2;
    public final static int ALLOW_LOWER = 3;
    public final static int ALLOW_LOWER_EQUAL = 4;
    public final static int ALLOW_HIGHER = 5;
    public final static int ALLOW_HIGHER_EQUAL = 6;

	public final static int COUNT_BIDIRECTIONAL_ONCE = 1;
	public final static int COUNT_BIDIRECTIONAL_TWICE = 2;

	private int countBidir;
	private int threshold;
    private int whichPart;
	private boolean relative;
	private int relativeMax;

	private int[] indices;

    protected Graph graph = null;
	protected Map degreeMap = null;
    private ArrayList listeners = new ArrayList();

    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.graph.EdgeCountFilter");

	public EdgeCountFilter(int countBidirectional, int threshold, int whichPart, boolean relative, int relativeMax) {
		this.countBidir = countBidirectional;
		setThreshold(threshold);
        this.whichPart = whichPart;
		this.relative = relative;
		this.relativeMax = relativeMax;
    }

    public void setThreshold(int threshold) {
		this.threshold = threshold;
        notifyListeners();
    }

	public int getThresholdValue() {
		if (relative && indices!=null) {
			return indices[threshold];
		} else {
			return threshold;
		}
	}

	// Returns percentile score
	protected void calculateIndicesAndMap() {
		logger.info("Calculating edge count map and indices");
		int[] values = new int[graph.nodes.size()];
		degreeMap = new HashMap();
		int num=0;
        for(Iterator i=graph.nodes.iterator(); i.hasNext(); num++){
            Node n = (Node) i.next();
			List inEdges = graph.inEdges(n);
			List outEdges = graph.outEdges(n);
			if(countBidir==COUNT_BIDIRECTIONAL_ONCE) {
				for(Iterator j=outEdges.iterator(); j.hasNext(); ){
					Edge e = (Edge) j.next();
					Edge reverse = new Edge(e.y, e.x);
					inEdges.remove(reverse);
				}
			}
			int degree = inEdges.size() + outEdges.size();

//			logger.info("Node "+n+" has degree "+degree);
			degreeMap.put(n, new Integer(degree));
			values[num] = degree;
		}

//		logger.info("Values:");
//		for(int i=0; i<values.length; i++) {
//			logger.info(""+values[i]);
//		}
		Arrays.sort(values);
//		logger.info("Sorted values:");
//		for(int i=0; i<values.length; i++) {
//			logger.info(""+values[i]);
//		}

		indices = new int[relativeMax+1];
		for(int i=0; i<relativeMax+1; i++) {
			int index = (int)Math.floor( (i*((double)1.0)/(relativeMax+1)) * values.length );
			indices[i] = values[index];
		}
		indices[0] = values[0]-1;
		indices[relativeMax] = values[values.length-1]+1;
//		logger.info("Result:");
//		for(int i=0; i<result.length; i++) {
//			logger.info(""+result[i]);
//		}
	}

	public boolean allowNode(Node node) {
        if (whichPart==DONT_FILTER) return true;
		if (graph==null) return true;
        Integer degree = (Integer)degreeMap.get(node);
        int compare = 0;
		if (relative) {
			if (degree==null) {
				logger.info("Node:   "+node);
				logger.info("Degree: "+degree);
				logger.info("Thresh: "+threshold);
				logger.info("Index:  "+indices[threshold]);
			}
			compare = degree.compareTo(new Integer(indices[threshold]));
		} else {
			compare = degree.compareTo(new Integer(threshold));
		}
        switch (whichPart) {
        case DONT_FILTER: return true;
        case ALLOW_EQUAL: return (compare==0);
        case ALLOW_UNEQUAL: return (compare!=0);
        case ALLOW_LOWER: return (compare<0);
        case ALLOW_LOWER_EQUAL: return (compare<=0);
        case ALLOW_HIGHER: return (compare>0);
        case ALLOW_HIGHER_EQUAL: return (compare>=0);
        }
        return true;
	}

    public boolean allowEdge(Edge edge) {
		return allowNode((Node)edge.x) && allowNode((Node)edge.y);
	}

	public void graphChanged(Graph graph) {
    	logger.info(this+": graph changed");
        this.graph = graph; 
		calculateIndicesAndMap();
    }

    public void addListener(GraphFilterListener listener) {
		logger.info("Added listener "+listener.toString()+" to edge count filter");
        listeners.add(listener);
    }

	public void clearListeners() {
		listeners = new ArrayList();
	}

    void notifyListeners() {
        for(Iterator i=listeners.iterator(); i.hasNext();) {
            ((GraphFilterListener) (i.next())).filterChanged(this);
        }
    }
}

