// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.graph;


import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.awt.print.*;
import java.text.*;
import java.util.*;
import java.util.logging.*;
import javax.swing.*;
import javax.swing.event.*;

import nl.tudelft.tbm.graph.*;


public class GraphComponent extends JPanel implements MouseListener, MouseMotionListener, Printable, GraphFilterListener {
    static final int NODE_SIZE = 7;
    static final int ARROW_SIZE = 6;

	static final int MAX_PROPERTY_LENGTH = 80;

    static final Paint EDGE_NORMAL_PAINT = Color.RED;
    static final Paint EDGE_DRAGGED_PAINT = Color.BLUE;

    public Graph graph; // private 
    protected Map nodeMap;
    protected Map edgeMap;
    private Random r;

    private int mouseDownX = 0;
    private int mouseDownY = 0;
    private boolean keepShowLabel;
    private boolean modified = false;
    private Popup popup = null;

    private EdgeComponent[] linkedEdges = null;
    private Paint[] linkedEdgesPaint = null;

    double factorX;
    double factorY;
    private boolean zoomFixed = true;
    private boolean showEdges = true;
	private int edgeStyle = EdgeComponent.STYLE_STRAIGHT;
	private String edgeStyleProperty = null;
	private String edgeColorProperty;
    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.graph");

    private ArrayList filters = new ArrayList();

    public GraphComponent(Graph graph) {
		this(graph, null, EdgeComponent.STYLE_CURVED);
	}

    public GraphComponent(Graph graph, String edgeColorProperty, int edgeStyle) {
        //	  super(); // layout manager
        this.graph = graph;
		this.edgeStyle = edgeStyle;
        nodeMap = new HashMap();
        edgeMap = new HashMap();

        //	  addMouseMotionListener(this);

        r = new Random();

        setBackground(Color.WHITE);
        factorX = 1.0;
        factorY = 1.0;
        calculateDisplayFactors();

        for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
            Node n = (Node) i.next();

            if (n.getX() == Node.POSITION_UNKNOWN) n.setX(r.nextInt(400) + 10);
            if (n.getY() == Node.POSITION_UNKNOWN) n.setY(r.nextInt(400) + 10);
            NodeComponent c = new NodeComponent(n, NODE_SIZE);

            //	      c.setLocation(r.nextInt(550)+20, r.nextInt(550)+20);
            c.addMouseListener(this);
            c.addMouseMotionListener(this);
            add(c);
            //	      c.setLocation((int) factorX*n.getX(), (int) factorY*n.getY());
            //	      c.setLocation((int) n.getX(), (int) n.getY());
            nodeMap.put(n, c);
        }

        for (Iterator i = graph.edges.iterator(); i.hasNext();) {
            Edge e = (Edge) i.next();
            NodeComponent from = (NodeComponent) nodeMap.get(e.x);
            NodeComponent to = (NodeComponent) nodeMap.get(e.y);
            EdgeComponent c = new EdgeComponent(e, from, to, ARROW_SIZE, edgeStyle);

            add(c);
            c.setLocation(0, 0);
            c.setSize(getSize());
            c.addMouseListener(this);
            edgeMap.put(e, c);
        }

		setEdgeColorProperty(edgeColorProperty);
        addMouseListener(this);
        modified = false;
    }

    public void setShowLabels(int showLabels) {
        for (int i = 0; i < getComponentCount(); i++) {
            Component c = getComponents()[i];
            if (c instanceof NodeComponent) {
                NodeComponent nc = (NodeComponent) c;
                nc.setShowLabel(showLabels);
            }
        }
    }

    public void setEdgeColorProperty(String property) {
		this.edgeColorProperty = property;
		if (property==null) {
			for (int i = 0; i < getComponentCount(); i++) {
				Component c = getComponents()[i];
				if (c instanceof EdgeComponent) {
					EdgeComponent ec = (EdgeComponent) c;
					ec.setPaint(EDGE_NORMAL_PAINT);
				}
			}
		} else {
			double min = Double.MAX_VALUE/2;
			double max = -min;

			for (Iterator i = graph.edges.iterator(); i.hasNext(); ) {
				Edge e = (Edge) i.next();
				try {
					double value = Double.parseDouble(e.getProperty(property));
					value = (value<1 ? 0 : Math.log(value));
					if(value<min) min=value;
					if(value>max) max=value;
				} catch (NumberFormatException ex) {
					logger.info("Warning: cannot parse edge colouring property "+property+" with value "+e.getProperty(property));
				}
			}
			double factor = 1/(max-min);
			for (Iterator i = graph.edges.iterator(); i.hasNext(); ) {
				Edge e = (Edge) i.next();
				try {
					double value = Double.parseDouble(e.getProperty(property));
					value = (value<1 ? 0 : Math.log(value));
					value = 2*(value-min)*factor;
					float red = (float) ((value>1)?(value-1):0);
					float green = (float) ((value<1)?1-value:0);
					Paint color = new Color(red, green, (float) 0);
					EdgeComponent c = (EdgeComponent) edgeMap.get(e);
					c.setPaint(color);
				} catch (NumberFormatException ex) {
					logger.info("Warning: cannot parse edge colouring property "+property+" with value "+e.getProperty(property));
				}
			}
		}
        repaint();
    }

    public void setShowEdges(boolean showEdges) {
        this.showEdges = showEdges;
        for (int i = 0; i < getComponentCount(); i++) {
            Component c = getComponents()[i];

            if (c instanceof EdgeComponent) {
                if (showEdges) {
                    c.show();
                } else {
                    c.hide();
                }
            }
        }
    }

	public void setEdgeStyle(int style) {
        this.edgeStyle = style;
        for (int i = 0; i < getComponentCount(); i++) {
            Component c = getComponents()[i];
            if (c instanceof EdgeComponent) {
				((EdgeComponent) c).setStyle(style);
            }
        }
		repaint();
	}

    public boolean isModified() {
        return modified;
    }

    public void setModified(boolean modified) {
        this.modified = modified;
    }

    public boolean isZoomFixed() {
        return zoomFixed;
    }

    public void setZoomFixed(boolean fixed) {
        this.zoomFixed = fixed;
    }

    public void mousePressed(MouseEvent e) {
        //	 logger.info("Mouse pressed (# of clicks: " + e.getClickCount() + ")");
        mouseDownX = e.getX();
        mouseDownY = e.getY();
        if (e.getComponent() instanceof NodeComponent) {
            NodeComponent c = (NodeComponent) e.getComponent();
            Vector edges = graph.inEdges(c.node);

            edges.addAll(graph.outEdges(c.node));
            linkedEdges = new EdgeComponent[edges.size()];
            linkedEdgesPaint = new Paint[edges.size()];
            int i = 0;

            for (Iterator it = edges.iterator(); it.hasNext(); i++) {
                linkedEdges[i] = (EdgeComponent) edgeMap.get(it.next());
                linkedEdgesPaint[i] = linkedEdges[i].getPaint();
                linkedEdges[i].setPaint(EDGE_DRAGGED_PAINT);
                // no need to repaint() the edge, since it will be moved anyway
                // and then repainted
            }
        }
    }

    public void mouseReleased(MouseEvent e) {
        if (linkedEdges!=null) {
            for (int i = 0; i < linkedEdges.length; i++) {
                linkedEdges[i].setPaint(linkedEdgesPaint[i]);
                linkedEdges[i].repaint();
            }
            linkedEdges = null;
            linkedEdgesPaint = null;
        }
        //	 logger.info("Mouse released (# of clicks: " + e.getClickCount() + ")");
        //	 if (e.getComponent() instanceof NodeComponent) {
        //	     NodeComponent c = (NodeComponent) e.getComponent();
        //	     repaint(); Not necessary any more since we update the edges at mouseDragged
        //	 }
    }

    public void mouseEntered(MouseEvent e) {

        /* Tooltips do it!
         logger.info("Mouse entered");
         if (e.getComponent() instanceof NodeComponent) {
         logger.info("Showing label");
         NodeComponent c = (NodeComponent) e.getComponent();
         keepShowLabel = c.showLabel;
         c.setShowLabel(true);
         }
         */
    }

    public void mouseExited(MouseEvent e) {

        /*
         logger.info("Mouse exited");
         if (e.getComponent() instanceof NodeComponent) {
         NodeComponent c = (NodeComponent) e.getComponent();
         c.setShowLabel(keepShowLabel);
         c.repaint();
         }
         */
    }

    void popupAttributeInfo(Component c, Attributed a, int x, int y) {
        //		 logger.info("Showing popup");
        StringBuffer info = new StringBuffer();
        info.append("Name: " + a.getName() + "\nLabel: " + a.getLabel());

        for (Iterator i = a.getPropertyNames().iterator(); i.hasNext();) {
            String prop = (String) i.next();
            String value = a.getProperty(prop);
			if (value.length()>MAX_PROPERTY_LENGTH) {
				value = value.substring(0, MAX_PROPERTY_LENGTH)+"...";
			}
            info.append("\n" + prop + ": " + value);
        }
        JTextArea contents = new JTextArea(info.toString());

        contents.setBackground(this.getBackground().darker());
        PopupFactory factory = PopupFactory.getSharedInstance();
        Dimension size = contents.getPreferredSize();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Point screenLocation = c.getLocationOnScreen();
        Point location = new Point();

        location.x = screenLocation.x + x; // + e.getX();
        location.y = screenLocation.y + y; // + e.getY() + 20;

        if (location.x + size.width > screenSize.width) {
            location.x -= size.width;
        }
        if (location.y + size.height > screenSize.height) {
            location.y -= (size.height + 20);
        }
        popup = factory.getPopup(this, contents, location.x, location.y);
        popup.show();
    }


    public void mouseClicked(MouseEvent e) {
        //	 logger.info("Mouse clicked (# of clicks: " + e.getClickCount() + ")");
        if (popup != null) {
            popup.hide();
            popup = null;
        }

        if (e.getComponent() instanceof NodeComponent) {
            NodeComponent c = (NodeComponent) e.getComponent();
            Node n = c.node;
            popupAttributeInfo(c, n, e.getX(), e.getY());
        } else if (e.getComponent() instanceof EdgeComponent) {
            EdgeComponent c = (EdgeComponent) e.getComponent();
            Edge edge = c.edge;
            popupAttributeInfo(c, edge, e.getX(), e.getY());
        } 

    }

    public void mouseDragged(MouseEvent e) {
        //	 logger.info("Mouse7 dragged (x,y: "+e.getX()+","+e.getY()+")");
        if (e.getComponent() instanceof NodeComponent) {
            NodeComponent c = (NodeComponent) e.getComponent();
            Node n = c.node;
            double moveX = (e.getX() - mouseDownX) / factorX;
            double moveY = (e.getY() - mouseDownY) / factorY;

            n.setPosition(n.getX() + moveX, n.getY() + moveY);
            //	     mouseDownX = 0;
            //	     mouseDownY = 0;
            c.setLocation((int) (factorX * n.getX()), (int) (factorY * n.getY()));
            //	     c.setLocation((int) n.getX(), (int) n.getY());
            c.repaint();
            for (int i = 0; i < linkedEdges.length; i++) {
                linkedEdges[i].repaint();
            }
            modified = true;
        }
    }

    public void mouseMoved(MouseEvent e) {
        //	 logger.info("Mouse moved (x,y: "+e.getX()+","+e.getY()+")");
    }

    public int print(Graphics g, PageFormat pf, int pageIndex) {
        if (pageIndex != 0) {
            return Printable.NO_SUCH_PAGE;
        }

        Graphics2D g2d = (Graphics2D) g;

        g2d.translate(pf.getImageableX(), pf.getImageableY());
        double scaleX = pf.getImageableWidth() / getWidth();
        double scaleY = pf.getImageableHeight() / getHeight();
        double scale = Math.min(scaleX, scaleY);

        g2d.scale(scale, scale);
        super.print(g2d);
        return Printable.PAGE_EXISTS;
    }

    public void paintComponent(Graphics g) {
        //	  logger.info("Painting graph");
        Graphics2D g2 = (Graphics2D) g;

        super.paintComponent(g);

        /* This is done by the Container stuff...
         logger.info("Painting nodes...");
         for(Iterator i=graph.nodes.iterator(); i.hasNext();) {
         Node n = (Node)i.next();
         NodeComponent c = (NodeComponent) nodeMap.get(n);
         c.paintComponent(g.create((int) (n.getX()-NODE_SIZE/2), (int) (n.getY()-NODE_SIZE/2), NODE_SIZE, NODE_SIZE));
        //	      n.setPosition(r.nextInt(550)+20, r.nextInt(550)+20);
        //	      c.setLocation(r.nextInt(550)+20, r.nextInt(550)+20);
        //	      add(c);
        //	      nodeMap.put(n, c);
        }
        */
    
        /*
         logger.info("Painting edges...");
         for(Iterator i=graph.edges.iterator(); i.hasNext();) {
         Edge e = (Edge)i.next();
         Node from = (Node) e.x;
         Node to = (Node) e.y;
        //	      logger.info("Drawing edge "+e.toString()+": x,y,x,y: "+from.getX()+","+from.getY()+","+to.getX()+","+to.getY());
        g2.draw(new Line2D.Double(from.getX()+NODE_SIZE/2,from.getY()+NODE_SIZE/2,to.getX()+NODE_SIZE/2,to.getY()+NODE_SIZE/2));
        }
        */
        //	  logger.info("Finished GraphComponent.paintComponent()");
    }

    public Dimension getPreferredSize() {
        //	  logger.info("Entering GraphComponent.getPreferredSize()");
        double maxX = 0;
        double maxY = 0;

        for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
            Node n = (Node) i.next();

            if (n.getX() > maxX) maxX = n.getX();
            if (n.getY() > maxY) maxY = n.getY();
        }
        return new Dimension((int) (maxX * factorX) + 2 * NODE_SIZE + 20, (int) (maxY * factorY) + 2 * NODE_SIZE + 20);
    }

    public Dimension getMinimumSize() {
        //	  logger.info("Entering GraphComponent.getMinumumSize()");
        return new Dimension(200, 150);
    }

    public void calculateDisplayFactors() {
        if (!zoomFixed) {
            double maxX = 0;
            double maxY = 0;

            for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
                Node n = (Node) i.next();

                if (n.getX() > maxX) maxX = n.getX();
                if (n.getY() > maxY) maxY = n.getY();
            }

            factorX = (getWidth() - 2 * NODE_SIZE - 50) / (maxX);
            factorY = (getHeight() - 2 * NODE_SIZE - 25) / (maxY);
        }
    }

    public void doLayout() {
        //	  logger.info("Entering GraphComponent.doLayout()");
        //	  super.doLayout();
        calculateDisplayFactors();
        for (int i = 0; i < getComponentCount(); i++) {
            Component c = getComponents()[i];

            if (c instanceof NodeComponent) {
                NodeComponent nc = (NodeComponent) c;
                Node n = nc.node;

                //		  logger.info("Placing a node at " + (int) (factorX*n.getX()) + ","+(int) (factorY*n.getY()) );
                c.setLocation((int) (factorX * n.getX()), (int) (factorY * n.getY()));
                //		  c.setLocation((int) (n.getX()-NODE_SIZE/2), (int) Math.round(n.getY()-NODE_SIZE/2));
                nc.doLayout();
            }
            if (c instanceof EdgeComponent) {
                //		  logger.info("Placing an edge");
                EdgeComponent ec = (EdgeComponent) c;

                c.setLocation(0, 0);
                c.setSize(getSize());
                ec.doLayout();
                //		  n.setLocation(r.nextInt(550)+20, r.nextInt(550)+20);
            }
        }
        repaint();
    }

    public void zoomFit() {
		double minX = Double.MAX_VALUE;
		double minY = Double.MAX_VALUE;

		for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
			Node n = (Node) i.next();
			if (n.getX() < minX) minX = n.getX();
			if (n.getY() < minY) minY = n.getY();
		}
		minX -= (factorX == 0.0) ? 50 : (50 / factorX);
		minY -= (factorY == 0.0) ? 25 : (25/ factorY);

		for (Iterator i = graph.nodes.iterator(); i.hasNext();) {
			Node n = (Node) i.next();
			n.setX(n.getX() - minX);
			n.setY(n.getY() - minY);
		}

        boolean save = zoomFixed;
		zoomFixed = false;
	    calculateDisplayFactors();
        zoomFixed = save;
        doLayout();
    }

    public void zoomIn() {
        factorX = 1.5 * factorX;
        factorY = 1.5 * factorY;
        doLayout();
    }

    public void zoomOut() {
        factorX = factorX / 1.5;
        factorY = factorY / 1.5;
        doLayout();
    }

    // GraphFilterListener methods
    public void addGraphFilter(GraphFilter filter) {
        if (!filters.contains(filter)) {
            filters.add(filter);
        }
        filterChanged(filter);
        filter.addListener(this);
    }

    public void addGraphFilters(java.util.List newFilters) {
		logger.info("Adding set of graph filters to graph component");
		for(Iterator i=newFilters.iterator(); i.hasNext(); ) {
			GraphFilter filter = (GraphFilter) i.next();
			if (!filters.contains(filter)) {
				filters.add(filter);
			}
			logger.info("Adding listener to filter");
			filter.addListener(this);
		}
		// HACK: only once, because filterChanged recalculated for all filters
        filterChanged(null);
    }



    public void removeGraphFilter(GraphFilter filter) {
        filters.remove(filter);
        filterChanged(null);
    }

    public void filterChanged(GraphFilter filter) {
        // Not very intelligent, recalculates visibility of all nodes
        // for every filter change
        for (int i = 0; i < getComponentCount(); i++) {
            Component c = getComponents()[i];

            if (c instanceof NodeComponent) {
                NodeComponent nc = (NodeComponent) c;
                Node node = nc.node;

                boolean allow = true;
                for(Iterator it=filters.iterator(); it.hasNext() && allow;) {
                    GraphFilter currentFilter = (GraphFilter) it.next();
                    allow = currentFilter.allowNode(node);
                }
                nc.setVisible(allow);
            }
            if (c instanceof EdgeComponent) {
                //		  logger.info("Placing an edge");
                EdgeComponent ec = (EdgeComponent) c;
                Edge edge = ec.edge;

                boolean allow = true;
                for(Iterator it=filters.iterator(); it.hasNext() && allow;) {
                    GraphFilter currentFilter = (GraphFilter) it.next();
                    allow = currentFilter.allowEdge(edge);
                }
                ec.setVisible(allow);
            }
        }
        repaint();
    }
}

