/**
 * Title:	Graph
 * Copyright:	Copyright (c) Jan-Pascal van Best
 * Email:	    janb@tbm.tudelft.nl
 * @author	    Jan-Pascal van Best
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.graph;


import java.util.*;
import java.lang.*;


/**
 * A node of a graph. As special properties, a <code>Node</code> has a
 * name, a label and a set of property-value pairs. A <code>Node</code>
 * also has a flag that denotes whether or not the node may be deleted.
 * This flag is used to pin down certain nodes when rearranging graphs
 * (for instance in the {@link Graph#removeDullNodes()} method of {@link Graph}).
 *
 * @author	    Jan-Pascal van Best
 * @version	1.0
 */

public interface Node extends Attributed {
    final static double POSITION_UNKNOWN = -65535;

    /**
     * Reads the <code>deletable</code> flag.
     * @return <code>true</code> if the node may be deleted, <code>false</code> if not.
     */
    public boolean isDeletable();

    /**
     * Sets the <code>deletable</code> flag.
     * @param deletable the new value of the <code>deletable</code> flag.
     */
    public void setDeletable(boolean deletable);

    public double getX();
    public double getY();
    public void setX(double x);
    public void setY(double y);
    public void setPosition(double x, double y);
}

