/**
 * Title:       Graph
 * Copyright:   Copyright (c) Jan-Pascal van Best
 * Email:	    janpascal@vanbest.org
 * Author: 	    Jan-Pascal van Best
 * Web:         http://www.tbm.tudelft.nl/webstaf/janb/software/graph/api/index.html
 **************************************************************************
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, version 2.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

package nl.tudelft.tbm.graph;


import java.util.*;
import java.lang.*;


/**
 * A pair of objects, usually representing an edge (or link) in a graph.
 * The nodes can be of any class type.
 *
 * @author 	    Jan-Pascal van Best
 * @version     1.0
 * @see         Graph
 */
public class Pair {

    /** the first element of the pair, or the origin of the edge */
    public Object x;

    /** the last element of the pair, or the destination of the edge */
    public Object y;

    /**
     * Constructs a new <code>Pair</code> with elements <code>x</code> and
     * <code>y</code>.
     * @param x the first element of the new <code>Pair</code>
     * @param y the last element of the new <code>Pair</code>
     */
    public Pair(Object x, Object y) {
        super();
        this.x = x;
        this.y = y;
    }

    /**
     * Returns a string representation of the pair
     * @return a string of the form <code>name1 -> name2</code>
     */
    public String toString() {
        return x.toString() + "->" + y.toString();
    }

    /**
     * Compares the specified object with this pair for equality. Returns
     * <code>true</code> if and only if the specified object is also a <code>Pair</code>,
     *          and both <code>x</code> and <code>y</code> of that object are
     *          <i>equal</i> to the elements of this pair.
     * @param obj the object to be checked for equality with this pair.
     * @return  <code>true</code> if the specified object is equal to this pair.
     */
    public boolean equals(Object obj) {
        boolean result = false;

        if (obj == null) result = false; 
        else {
            if (obj instanceof Pair) {
                Pair p = (Pair) obj;

                result = (x.equals(p.x) && y.equals(p.y));
            } else {
                result = false;
            }
        }
        return result;
    }

    /**
     * Returns the hash code for this pair. The hash code of a Pair is defined to
     * be <code>x.hashCode() + y.hashCode()</code>.
     * @return the hash code value for this pair
     * @see java.lang.Object#hashCode()
     * @see java.lang.Object#equals(Object)
     * @see #equals(Object)
     */
    public int hashCode() {
        return (x.hashCode() + y.hashCode());
    }
}

