// This progarm is intended to query and display  from remote MySql database server
// that holds the TTM database file.

package nl.tudelft.tbm.graph;

import java.text.*;
import java.util.*;
import java.util.logging.*;

public abstract class PropertyFilter implements GraphFilter {
    public final static int FILTER_TYPE_STRING = 0;
    public final static int FILTER_TYPE_DOUBLE = 1;

    public final static int DONT_FILTER = 0;
    public final static int ALLOW_EQUAL = 1;
    public final static int ALLOW_UNEQUAL = 2;
    public final static int ALLOW_LOWER = 3;
    public final static int ALLOW_LOWER_EQUAL = 4;
    public final static int ALLOW_HIGHER = 5;
    public final static int ALLOW_HIGHER_EQUAL = 6;

    private String property;
    private int filterType;
    private String thresholdString;
    private double thresholdDouble;
	private int thresholdInt;
    private int whichPart;
	private boolean relative;
	private int relativeMax;

	private double[] indices;

    protected Graph graph = null;
    private ArrayList listeners = new ArrayList();

    private static Logger logger = Logger.getLogger("nl.tudelft.tbm.graph.PropertyFilter");

	public PropertyFilter(String property, int filterType, String threshold, int whichPart, boolean relative, int relativeMax) throws NumberFormatException {
        this.property = property;
        this.filterType = filterType;
        this.whichPart = whichPart;
		this.relative = relative;
		this.relativeMax = relativeMax;
        setThreshold(threshold);
    }

    public void setThreshold(String threshold) {
        switch (filterType) {
        case FILTER_TYPE_STRING:
            this.thresholdString = threshold;
            break;
        case FILTER_TYPE_DOUBLE:
            this.thresholdDouble = Double.parseDouble(threshold);
			this.thresholdInt = (int) Math.round(thresholdDouble);
            break;
        }
        notifyListeners();
    }

    protected boolean doTest(Attributed element) {
        if (whichPart==DONT_FILTER) return true;
		if (graph==null) return true;
        String value = element.getProperty(property);
        int compare = 0;
        switch (filterType) {
        case FILTER_TYPE_STRING:
            compare = value.compareTo(thresholdString);
            break;
        case FILTER_TYPE_DOUBLE:
            try {
                double doubleValue = Double.parseDouble(value);
				if (relative) {
					compare = Double.compare(doubleValue, indices[thresholdInt]);
				} else {
	                compare = Double.compare(doubleValue, thresholdDouble);
				}
            } catch (NumberFormatException e) {
                compare = 0;
            }
            break;
        }
        switch (whichPart) {
        case DONT_FILTER: return true;
        case ALLOW_EQUAL: return (compare==0);
        case ALLOW_UNEQUAL: return (compare!=0);
        case ALLOW_LOWER: return (compare<0);
        case ALLOW_LOWER_EQUAL: return (compare<=0);
        case ALLOW_HIGHER: return (compare>0);
        case ALLOW_HIGHER_EQUAL: return (compare>=0);
        }
        return true;
    }

	public double getThresholdValue() {
		if (relative && indices!=null) {
			return indices[thresholdInt];
		} else {
			return thresholdDouble;
		}
	}


	// Returns percentile score
	protected double[] calculateIndices(Collection items) {
		logger.info("Calculating indices");
		double[] values = new double[items.size()];
		int num=0;
        for(Iterator i=items.iterator(); i.hasNext(); num++){
            Attributed it = (Attributed) i.next();
	        String value = it.getProperty(property);
			double doubleValue;
			try {
				doubleValue = Double.parseDouble(value);
			} catch (NumberFormatException ex) {
				doubleValue = 0;
			}
			values[num] = doubleValue;
		}
//		logger.info("Values:");
//		for(int i=0; i<values.length; i++) {
//			logger.info(""+values[i]);
//		}
		Arrays.sort(values);
//		logger.info("Sorted values:");
//		for(int i=0; i<values.length; i++) {
//			logger.info(""+values[i]);
//		}

		double[] result = new double[relativeMax+1];
		for(int i=0; i<relativeMax+1; i++) {
			int index = (int)Math.floor( (i*((double)1.0)/(relativeMax+1)) * values.length );
			result[i] = values[index];
		}
		result[0] = values[0]-1;
		result[relativeMax] = values[values.length-1]+1;
//		logger.info("Result:");
//		for(int i=0; i<result.length; i++) {
//			logger.info(""+result[i]);
//		}
		return result;
	}

	protected void setIndices(double[] indices) {
		this.indices = indices;
	}

    public void graphChanged(Graph graph) {
        this.graph = graph; 
    }

    public void addListener(GraphFilterListener listener) {
		logger.info("Added listener "+listener.toString()+" to property filter");
        listeners.add(listener);
    }

	public void clearListeners() {
		listeners = new ArrayList();
	}

    void notifyListeners() {
        for(Iterator i=listeners.iterator(); i.hasNext();) {
            ((GraphFilterListener) (i.next())).filterChanged(this);
        }
    }
}

